"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EufySecurity = void 0;
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const ts_log_1 = require("ts-log");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const api_1 = require("./http/api");
const station_1 = require("./http/station");
const types_1 = require("./http/types");
const service_1 = require("./push/service");
const device_1 = require("./http/device");
const types_2 = require("./p2p/types");
const utils_1 = require("./utils");
const error_1 = require("./error");
const _1 = require(".");
const error_2 = require("./http/error");
const push_1 = require("./push");
class EufySecurity extends tiny_typed_emitter_1.TypedEmitter {
    constructor(config, log = ts_log_1.dummyLogger) {
        super();
        this.stations = {};
        this.devices = {};
        this.P2P_REFRESH_INTERVAL_MIN = 720;
        this.cameraMaxLivestreamSeconds = 30;
        this.cameraStationLivestreamTimeout = new Map();
        this.cameraCloudLivestreamTimeout = new Map();
        this.pushCloudRegistered = false;
        this.pushCloudChecked = false;
        this.persistentData = {
            api_base: "",
            cloud_token: "",
            cloud_token_expiration: 0,
            openudid: "",
            serial_number: "",
            push_credentials: undefined,
            push_persistentIds: [],
            login_hash: "",
            version: ""
        };
        this.connected = false;
        this.refreshEufySecurityP2PTimeout = {};
        this.config = config;
        this.log = log;
        if (this.config.country === undefined) {
            this.config.country = "US";
        }
        if (this.config.language === undefined) {
            this.config.language = "en";
        }
        if (this.config.trustedDeviceName === undefined) {
            this.config.trustedDeviceName = "eufyclient";
        }
        if (this.config.eventDurationSeconds === undefined) {
            this.config.eventDurationSeconds = 10;
        }
        if (this.config.p2pConnectionSetup === undefined) {
            this.config.p2pConnectionSetup = types_2.P2PConnectionType.PREFER_LOCAL;
        }
        else if (!Object.values(types_2.P2PConnectionType).includes(this.config.p2pConnectionSetup)) {
            this.config.p2pConnectionSetup = types_2.P2PConnectionType.PREFER_LOCAL;
        }
        if (this.config.pollingIntervalMinutes === undefined) {
            this.config.pollingIntervalMinutes = 10;
        }
        if (this.config.acceptInvitations === undefined) {
            this.config.acceptInvitations = false;
        }
        if (this.config.persistentDir === undefined) {
            this.config.persistentDir = path_1.default.resolve(__dirname, "..");
        }
        else if (!fs_extra_1.default.existsSync(this.config.persistentDir)) {
            this.config.persistentDir = path_1.default.resolve(__dirname, "..");
        }
        this.persistentFile = path_1.default.join(this.config.persistentDir, "persistent.json");
        try {
            if (fs_extra_1.default.statSync(this.persistentFile).isFile()) {
                const fileContent = fs_extra_1.default.readFileSync(this.persistentFile, "utf8");
                this.persistentData = JSON.parse(fileContent);
            }
        }
        catch (err) {
            this.log.debug("No stored data from last exit found");
        }
        try {
            if (this.persistentData.version !== _1.libVersion) {
                const currentVersion = Number.parseFloat((0, utils_1.removeLastChar)(_1.libVersion, "."));
                const previousVersion = this.persistentData.version !== "" && this.persistentData.version !== undefined ? Number.parseFloat((0, utils_1.removeLastChar)(this.persistentData.version, ".")) : 0;
                this.log.debug("Handling of adapter update", { currentVersion: currentVersion, previousVersion: previousVersion });
                if (previousVersion < currentVersion) {
                    this.persistentData = (0, utils_1.handleUpdate)(this.persistentData, this.log, previousVersion);
                    this.persistentData.version = _1.libVersion;
                    this.writePersistentData();
                }
            }
        }
        catch (error) {
            this.log.error("Handling update - Error:", error);
        }
        this.api = new api_1.HTTPApi(this.config.username, this.config.password, this.log);
        this.api.setCountry(this.config.country);
        this.api.setLanguage(this.config.language);
        this.api.setPhoneModel(this.config.trustedDeviceName);
        this.api.on("hubs", (hubs) => this.handleHubs(hubs));
        this.api.on("devices", (devices) => this.handleDevices(devices));
        this.api.on("close", () => this.onAPIClose());
        this.api.on("connect", () => this.onAPIConnect());
        this.api.on("captcha request", (id, captcha) => this.onCaptchaRequest(id, captcha));
        if (this.persistentData.login_hash && this.persistentData.login_hash != "") {
            this.log.debug("Load previous login_hash:", this.persistentData.login_hash);
            if ((0, utils_1.md5)(`${this.config.username}:${this.config.password}`) != this.persistentData.login_hash) {
                this.log.info("Authentication properties changed, invalidate saved cloud token.");
                this.persistentData.cloud_token = "";
                this.persistentData.cloud_token_expiration = 0;
                this.persistentData.api_base = "";
            }
        }
        else {
            this.persistentData.cloud_token = "";
            this.persistentData.cloud_token_expiration = 0;
        }
        if (this.persistentData.api_base && this.persistentData.api_base != "") {
            this.log.debug("Load previous api_base:", this.persistentData.api_base);
            this.api.setAPIBase(this.persistentData.api_base);
        }
        if (this.persistentData.cloud_token && this.persistentData.cloud_token != "") {
            this.log.debug("Load previous token:", { token: this.persistentData.cloud_token, tokenExpiration: this.persistentData.cloud_token_expiration });
            this.api.setToken(this.persistentData.cloud_token);
            this.api.setTokenExpiration(new Date(this.persistentData.cloud_token_expiration));
        }
        if (!this.persistentData.openudid || this.persistentData.openudid == "") {
            this.persistentData.openudid = (0, utils_1.generateUDID)();
            this.log.debug("Generated new openudid:", this.persistentData.openudid);
        }
        this.api.setOpenUDID(this.persistentData.openudid);
        if (!this.persistentData.serial_number || this.persistentData.serial_number == "") {
            this.persistentData.serial_number = (0, utils_1.generateSerialnumber)(12);
            this.log.debug("Generated new serial_number:", this.persistentData.serial_number);
        }
        this.api.setSerialNumber(this.persistentData.serial_number);
        this.pushService = new service_1.PushNotificationService(this.log);
        this.pushService.on("connect", async (token) => {
            this.pushCloudRegistered = await this.api.registerPushToken(token);
            this.pushCloudChecked = await this.api.checkPushToken();
            //TODO: Retry if failed with max retry to not lock account
            if (this.pushCloudRegistered && this.pushCloudChecked) {
                this.log.info("Push notification connection successfully established");
                this.emit("push connect");
            }
            else {
                this.log.info("Push notification connection closed");
                this.emit("push close");
            }
        });
        this.pushService.on("credential", (credentials) => {
            this.savePushCredentials(credentials);
        });
        this.pushService.on("message", (message) => this.onPushMessage(message));
        this.pushService.on("close", () => {
            this.log.info("Push notification connection closed");
            this.emit("push close");
        });
    }
    getPushService() {
        return this.pushService;
    }
    addStation(station) {
        const serial = station.getSerial();
        if (serial && !Object.keys(this.stations).includes(serial)) {
            this.stations[serial] = station;
            this.emit("station added", station);
            station.setConnectionType(this.config.p2pConnectionSetup);
            station.connect();
        }
        else {
            throw new error_1.DuplicateStationError(`Station with this serial ${station.getSerial()} exists already and couldn't be added again!`);
        }
    }
    removeStation(station) {
        const serial = station.getSerial();
        if (serial && Object.keys(this.stations).includes(serial)) {
            delete this.stations[serial];
            station.removeAllListeners();
            if (station.isConnected())
                station.close();
            this.emit("station removed", station);
        }
        else {
            throw new error_1.StationNotFoundError(`Station with this serial ${station.getSerial()} doesn't exists and couldn't be removed!`);
        }
    }
    updateStation(hub) {
        if (Object.keys(this.stations).includes(hub.station_sn)) {
            this.stations[hub.station_sn].update(hub);
            if (!this.stations[hub.station_sn].isConnected() && !this.stations[hub.station_sn].isEnergySavingDevice()) {
                this.stations[hub.station_sn].setConnectionType(this.config.p2pConnectionSetup);
                this.stations[hub.station_sn].connect();
            }
        }
        else {
            throw new error_1.StationNotFoundError(`Station with this serial ${hub.station_sn} doesn't exists and couldn't be updated!`);
        }
    }
    addDevice(device) {
        const serial = device.getSerial();
        if (serial && !Object.keys(this.devices).includes(serial)) {
            this.devices[serial] = device;
            this.emit("device added", device);
        }
        else {
            throw new error_1.DuplicateDeviceError(`Device with this serial ${device.getSerial()} exists already and couldn't be added again!`);
        }
    }
    removeDevice(device) {
        const serial = device.getSerial();
        if (serial && Object.keys(this.devices).includes(serial)) {
            delete this.devices[serial];
            device.removeAllListeners();
            this.emit("device removed", device);
        }
        else {
            throw new error_1.DuplicateDeviceError(`Device with this serial ${device.getSerial()} doesn't exists and couldn't be removed!`);
        }
    }
    updateDevice(device) {
        if (Object.keys(this.devices).includes(device.device_sn))
            this.devices[device.device_sn].update(device);
        else
            throw new error_1.DeviceNotFoundError(`Device with this serial ${device.device_sn} doesn't exists and couldn't be updated!`);
    }
    getDevices() {
        const arr = [];
        Object.keys(this.devices).forEach((serialNumber) => {
            arr.push(this.devices[serialNumber]);
        });
        return arr;
    }
    getDevice(deviceSN) {
        if (Object.keys(this.devices).includes(deviceSN))
            return this.devices[deviceSN];
        throw new error_1.DeviceNotFoundError(`Device with this serial ${deviceSN} doesn't exists!`);
    }
    getStationDevice(stationSN, channel) {
        for (const device of Object.values(this.devices)) {
            if ((device.getStationSerial() === stationSN && device.getChannel() === channel) || (device.getStationSerial() === stationSN && device.getSerial() === stationSN)) {
                return device;
            }
        }
        throw new error_1.DeviceNotFoundError(`No device with channel ${channel} found on station with serial number: ${stationSN}!`);
    }
    getStations() {
        const arr = [];
        Object.keys(this.stations).forEach((serialNumber) => {
            arr.push(this.stations[serialNumber]);
        });
        return arr;
    }
    getStation(stationSN) {
        if (Object.keys(this.stations).includes(stationSN))
            return this.stations[stationSN];
        throw new error_1.StationNotFoundError(`No station with serial number: ${stationSN}!`);
    }
    getApi() {
        return this.api;
    }
    async connectToStation(stationSN, p2pConnectionType = types_2.P2PConnectionType.PREFER_LOCAL) {
        if (Object.keys(this.stations).includes(stationSN)) {
            this.stations[stationSN].setConnectionType(p2pConnectionType);
            this.stations[stationSN].connect();
        }
        else
            throw new error_1.StationNotFoundError(`No station with this serial number: ${stationSN}!`);
    }
    isStationConnected(stationSN) {
        if (Object.keys(this.stations).includes(stationSN))
            return this.stations[stationSN].isConnected();
        throw new error_1.StationNotFoundError(`No station with this serial number: ${stationSN}!`);
    }
    isStationEnergySavingDevice(stationSN) {
        if (Object.keys(this.stations).includes(stationSN))
            return this.stations[stationSN].isEnergySavingDevice();
        throw new error_1.StationNotFoundError(`No station with this serial number: ${stationSN}!`);
    }
    handleHubs(hubs) {
        this.log.debug("Got hubs:", hubs);
        const stationsSNs = Object.keys(this.stations);
        const newStationsSNs = Object.keys(hubs);
        for (const hub of Object.values(hubs)) {
            if (stationsSNs.includes(hub.station_sn)) {
                this.updateStation(hub);
            }
            else {
                const station = new station_1.Station(this.api, hub);
                station.on("connect", (station) => this.onStationConnect(station));
                station.on("close", (station) => this.onStationClose(station));
                station.on("raw device property changed", (deviceSN, params) => this.updateDeviceProperties(deviceSN, params));
                station.on("livestream start", (station, channel, metadata, videostream, audiostream) => this.onStartStationLivestream(station, channel, metadata, videostream, audiostream));
                station.on("livestream stop", (station, channel) => this.onStopStationLivestream(station, channel));
                station.on("download start", (station, channel, metadata, videoStream, audioStream) => this.onStationStartDownload(station, channel, metadata, videoStream, audioStream));
                station.on("download finish", (station, channel) => this.onStationFinishDownload(station, channel));
                station.on("command result", (station, result) => this.onStationCommandResult(station, result));
                station.on("guard mode", (station, guardMode) => this.onStationGuardMode(station, guardMode));
                station.on("current mode", (station, currentMode) => this.onStationCurrentMode(station, currentMode));
                station.on("rtsp livestream start", (station, channel) => this.onStartStationRTSPLivestream(station, channel));
                station.on("rtsp livestream stop", (station, channel) => this.onStopStationRTSPLivestream(station, channel));
                station.on("rtsp url", (station, channel, value, modified) => this.onStationRtspUrl(station, channel, value, modified));
                station.on("property changed", (station, name, value) => this.onStationPropertyChanged(station, name, value));
                station.on("property renewed", (station, name, value) => this.onStationPropertyRenewed(station, name, value));
                station.on("raw property changed", (station, type, value, modified) => this.onStationRawPropertyChanged(station, type, value, modified));
                station.on("raw property renewed", (station, type, value, modified) => this.onStationRawPropertyRenewed(station, type, value, modified));
                station.on("alarm event", (station, alarmEvent) => this.onStationAlarmEvent(station, alarmEvent));
                station.on("runtime state", (station, channel, batteryLevel, temperature, modified) => this.onStationRuntimeState(station, channel, batteryLevel, temperature, modified));
                station.on("charging state", (station, channel, chargeType, batteryLevel, modified) => this.onStationChargingState(station, channel, chargeType, batteryLevel, modified));
                station.on("wifi rssi", (station, channel, rssi, modified) => this.onStationWifiRssi(station, channel, rssi, modified));
                station.on("floodlight manual switch", (station, channel, enabled, modified) => this.onFloodlightManualSwitch(station, channel, enabled, modified));
                this.addStation(station);
            }
        }
        for (const stationSN of stationsSNs) {
            if (!newStationsSNs.includes(stationSN)) {
                this.removeStation(this.getStation(stationSN));
            }
        }
    }
    onStationConnect(station) {
        this.emit("station connect", station);
        if (station.getDeviceType() !== types_1.DeviceType.DOORBELL) {
            station.getCameraInfo().catch(error => {
                this.log.error(`Error during station ${station.getSerial()} p2p data refreshing`, error);
            });
            if (this.refreshEufySecurityP2PTimeout[station.getSerial()] !== undefined)
                clearTimeout(this.refreshEufySecurityP2PTimeout[station.getSerial()]);
            this.refreshEufySecurityP2PTimeout[station.getSerial()] = setTimeout(() => {
                station.getCameraInfo().catch(error => {
                    this.log.error(`Error during station ${station.getSerial()} p2p data refreshing`, error);
                });
            }, this.P2P_REFRESH_INTERVAL_MIN * 60 * 1000);
        }
    }
    onStationClose(station) {
        this.emit("station close", station);
        try {
            for (const device_sn of this.cameraStationLivestreamTimeout.keys()) {
                const device = this.getDevice(device_sn);
                if (device !== null && device.getStationSerial() === station.getSerial()) {
                    clearTimeout(this.cameraStationLivestreamTimeout.get(device_sn));
                    this.cameraStationLivestreamTimeout.delete(device_sn);
                }
            }
        }
        catch (error) {
            this.log.error(`Station ${station.getSerial()} - Error:`, error);
        }
    }
    handleDevices(devices) {
        this.log.debug("Got devices:", devices);
        const deviceSNs = Object.keys(this.devices);
        const newDeviceSNs = Object.keys(devices);
        for (const device of Object.values(devices)) {
            if (deviceSNs.includes(device.device_sn)) {
                this.updateDevice(device);
            }
            else {
                let new_device;
                if (device_1.Device.isIndoorCamera(device.device_type)) {
                    new_device = new device_1.IndoorCamera(this.api, device);
                }
                else if (device_1.Device.isSoloCameras(device.device_type)) {
                    new_device = new device_1.SoloCamera(this.api, device);
                }
                else if (device_1.Device.isBatteryDoorbell(device.device_type) || device_1.Device.isBatteryDoorbell2(device.device_type)) {
                    new_device = new device_1.BatteryDoorbellCamera(this.api, device);
                }
                else if (device_1.Device.isWiredDoorbell(device.device_type)) {
                    new_device = new device_1.DoorbellCamera(this.api, device);
                }
                else if (device_1.Device.isFloodLight(device.device_type)) {
                    new_device = new device_1.FloodlightCamera(this.api, device);
                }
                else if (device_1.Device.isCamera(device.device_type)) {
                    new_device = new device_1.Camera(this.api, device);
                }
                else if (device_1.Device.isLock(device.device_type)) {
                    new_device = new device_1.Lock(this.api, device);
                }
                else if (device_1.Device.isMotionSensor(device.device_type)) {
                    new_device = new device_1.MotionSensor(this.api, device);
                }
                else if (device_1.Device.isEntrySensor(device.device_type)) {
                    new_device = new device_1.EntrySensor(this.api, device);
                }
                else if (device_1.Device.isKeyPad(device.device_type)) {
                    new_device = new device_1.Keypad(this.api, device);
                }
                else {
                    new_device = new device_1.UnknownDevice(this.api, device);
                }
                new_device.on("property changed", (device, name, value) => this.onDevicePropertyChanged(device, name, value));
                new_device.on("property renewed", (device, name, value) => this.onDevicePropertyRenewed(device, name, value));
                new_device.on("raw property changed", (device, type, value, modified) => this.onDeviceRawPropertyChanged(device, type, value, modified));
                new_device.on("raw property renewed", (device, type, value, modified) => this.onDeviceRawPropertyRenewed(device, type, value, modified));
                new_device.on("crying detected", (device, state) => this.onDeviceCryingDetected(device, state));
                new_device.on("sound detected", (device, state) => this.onDeviceSoundDetected(device, state));
                new_device.on("pet detected", (device, state) => this.onDevicePetDetected(device, state));
                new_device.on("motion detected", (device, state) => this.onDeviceMotionDetected(device, state));
                new_device.on("person detected", (device, state, person) => this.onDevicePersonDetected(device, state, person));
                new_device.on("rings", (device, state) => this.onDeviceRings(device, state));
                new_device.on("locked", (device, state) => this.onDeviceLocked(device, state));
                new_device.on("open", (device, state) => this.onDeviceOpen(device, state));
                new_device.on("ready", (device) => this.onDeviceReady(device));
                this.addDevice(new_device);
            }
        }
        for (const deviceSN of deviceSNs) {
            if (!newDeviceSNs.includes(deviceSN)) {
                this.removeDevice(this.getDevice(deviceSN));
            }
        }
    }
    async refreshCloudData() {
        if (this.config.acceptInvitations) {
            await this.processInvitations().catch(error => {
                this.log.error("Error in processing invitations", error);
            });
        }
        await this.api.updateDeviceInfo().catch(error => {
            this.log.error("Error during API data refreshing", error);
        });
        if (this.refreshEufySecurityCloudTimeout !== undefined)
            clearTimeout(this.refreshEufySecurityCloudTimeout);
        this.refreshEufySecurityCloudTimeout = setTimeout(() => { this.refreshCloudData(); }, this.config.pollingIntervalMinutes * 60 * 1000);
    }
    close() {
        for (const device_sn of this.cameraStationLivestreamTimeout.keys()) {
            this.stopStationLivestream(device_sn);
        }
        for (const device_sn of this.cameraCloudLivestreamTimeout.keys()) {
            this.stopCloudLivestream(device_sn);
        }
        if (this.refreshEufySecurityCloudTimeout !== undefined)
            clearTimeout(this.refreshEufySecurityCloudTimeout);
        Object.keys(this.refreshEufySecurityP2PTimeout).forEach(station_sn => {
            clearTimeout(this.refreshEufySecurityP2PTimeout[station_sn]);
        });
        this.savePushPersistentIds();
        this.pushService.close();
        Object.values(this.stations).forEach(station => {
            station.close();
        });
        Object.values(this.devices).forEach(device => {
            device.destroy();
        });
        if (this.connected)
            this.emit("close");
        this.connected = false;
    }
    setCameraMaxLivestreamDuration(seconds) {
        this.cameraMaxLivestreamSeconds = seconds;
    }
    getCameraMaxLivestreamDuration() {
        return this.cameraMaxLivestreamSeconds;
    }
    async registerPushNotifications(credentials, persistentIds) {
        if (credentials)
            this.pushService.setCredentials(credentials);
        if (persistentIds)
            this.pushService.setPersistentIds(persistentIds);
        this.pushService.open();
    }
    async connect(verifyCodeOrCaptcha, captchaId) {
        let retries = 0;
        while (true) {
            switch (await this.api.authenticate(verifyCodeOrCaptcha, captchaId)) {
                case types_1.AuthResult.CAPTCHA_NEEDED:
                    return false;
                case types_1.AuthResult.SEND_VERIFY_CODE:
                    this.saveAPIBase();
                    this.emit("tfa request");
                    return false;
                case types_1.AuthResult.RENEW:
                    this.log.debug("Renew token");
                    break;
                case types_1.AuthResult.ERROR:
                    this.log.error("Token error");
                    return false;
                case types_1.AuthResult.OK:
                    if (verifyCodeOrCaptcha && !captchaId) {
                        return await this.api.addTrustDevice(verifyCodeOrCaptcha);
                    }
                    return true;
            }
            if (retries > 2) {
                this.log.error("Max connect attempts reached, interrupt");
                return false;
            }
            else {
                retries += 1;
            }
        }
    }
    getPushPersistentIds() {
        return this.pushService.getPersistentIds();
    }
    updateDeviceProperties(deviceSN, values) {
        const device = this.getDevice(deviceSN);
        device.updateRawProperties(values);
    }
    onAPIClose() {
        this.connected = false;
        this.emit("close");
    }
    async onAPIConnect() {
        this.connected = true;
        this.emit("connect");
        this.saveAPIBase();
        try {
            const token_expiration = this.api.getTokenExpiration();
            const trusted_token_expiration = this.api.getTrustedTokenExpiration();
            if ((token_expiration === null || token_expiration === void 0 ? void 0 : token_expiration.getTime()) !== trusted_token_expiration.getTime()) {
                const trusted_devices = await this.api.listTrustDevice();
                trusted_devices.forEach(trusted_device => {
                    if (trusted_device.is_current_device === 1) {
                        this.api.setTokenExpiration(trusted_token_expiration);
                        this.log.debug(`This device is trusted. Token expiration extended to ${trusted_token_expiration})`);
                    }
                });
            }
        }
        catch (error) {
            this.log.error("Trusted devices - Error:", error);
        }
        this.saveCloudToken();
        this.registerPushNotifications(this.persistentData.push_credentials, this.persistentData.push_persistentIds);
        this.refreshCloudData();
    }
    async startStationLivestream(deviceSN) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        if (!device.hasCommand(types_1.CommandName.DeviceStartLivestream))
            throw new error_1.NotSupportedError(`This functionality is not implemented or supported by ${device.getSerial()}`);
        const camera = device;
        if (!station.isLiveStreaming(camera)) {
            station.startLivestream(camera);
            this.cameraStationLivestreamTimeout.set(deviceSN, setTimeout(() => {
                this.log.info(`Stopping the station stream for the device ${deviceSN}, because we have reached the configured maximum stream timeout (${this.cameraMaxLivestreamSeconds} seconds)`);
                this.stopStationLivestream(deviceSN);
            }, this.cameraMaxLivestreamSeconds * 1000));
        }
        else {
            this.log.warn(`The station stream for the device ${deviceSN} cannot be started, because it is already streaming!`);
        }
    }
    async startCloudLivestream(deviceSN) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        if (!device.hasCommand(types_1.CommandName.DeviceStartLivestream))
            throw new error_1.NotSupportedError(`This functionality is not implemented or supported by ${device.getSerial()}`);
        const camera = device;
        if (!camera.isStreaming()) {
            const url = await camera.startStream();
            if (url !== "") {
                this.cameraCloudLivestreamTimeout.set(deviceSN, setTimeout(() => {
                    this.log.info(`Stopping the station stream for the device ${deviceSN}, because we have reached the configured maximum stream timeout (${this.cameraMaxLivestreamSeconds} seconds)`);
                    this.stopCloudLivestream(deviceSN);
                }, this.cameraMaxLivestreamSeconds * 1000));
                this.emit("cloud livestream start", station, camera, url);
            }
            else {
                this.log.error(`Failed to start cloud stream for the device ${deviceSN}`);
            }
        }
        else {
            this.log.warn(`The cloud stream for the device ${deviceSN} cannot be started, because it is already streaming!`);
        }
    }
    async stopStationLivestream(deviceSN) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        if (!device.hasCommand(types_1.CommandName.DeviceStopLivestream))
            throw new error_1.NotSupportedError(`This functionality is not implemented or supported by ${device.getSerial()}`);
        if (station.isConnected() && station.isLiveStreaming(device)) {
            await station.stopLivestream(device);
        }
        else {
            this.log.warn(`The station stream for the device ${deviceSN} cannot be stopped, because it isn't streaming!`);
        }
        const timeout = this.cameraStationLivestreamTimeout.get(deviceSN);
        if (timeout) {
            clearTimeout(timeout);
            this.cameraStationLivestreamTimeout.delete(deviceSN);
        }
    }
    async stopCloudLivestream(deviceSN) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        if (!device.hasCommand(types_1.CommandName.DeviceStopLivestream))
            throw new error_1.NotSupportedError(`This functionality is not implemented or supported by ${device.getSerial()}`);
        const camera = device;
        if (camera.isStreaming()) {
            await camera.stopStream();
            this.emit("cloud livestream stop", station, camera);
        }
        else {
            this.log.warn(`The cloud stream for the device ${deviceSN} cannot be stopped, because it isn't streaming!`);
        }
        const timeout = this.cameraCloudLivestreamTimeout.get(deviceSN);
        if (timeout) {
            clearTimeout(timeout);
            this.cameraCloudLivestreamTimeout.delete(deviceSN);
        }
    }
    writePersistentData() {
        this.persistentData.login_hash = (0, utils_1.md5)(`${this.config.username}:${this.config.password}`);
        try {
            fs_extra_1.default.writeFileSync(this.persistentFile, JSON.stringify(this.persistentData));
        }
        catch (error) {
            this.log.error("Error:", error);
        }
    }
    saveAPIBase() {
        if (this.persistentData.api_base !== this.api.getAPIBase()) {
            this.persistentData.api_base = this.api.getAPIBase();
            this.writePersistentData();
        }
    }
    saveCloudToken() {
        const token = this.api.getToken();
        const token_expiration = this.api.getTokenExpiration();
        if (!!token && !!token_expiration) {
            this.log.debug("Save cloud token and token expiration", { token: token, tokenExpiration: token_expiration });
            this.persistentData.cloud_token = token;
            this.persistentData.cloud_token_expiration = token_expiration.getTime();
            this.writePersistentData();
        }
    }
    savePushCredentials(credentials) {
        this.persistentData.push_credentials = credentials;
        this.writePersistentData();
    }
    savePushPersistentIds() {
        this.persistentData.push_persistentIds = this.getPushPersistentIds();
        this.writePersistentData();
    }
    getVersion() {
        return _1.libVersion;
    }
    isPushConnected() {
        return this.pushService.isConnected();
    }
    isConnected() {
        return this.connected;
    }
    async processInvitations() {
        const invites = await this.api.getInvites();
        if (Object.keys(invites).length > 0) {
            const confirmInvites = [];
            for (const invite of Object.values(invites)) {
                const devices = [];
                invite.devices.forEach(device => {
                    devices.push(device.device_sn);
                });
                if (devices.length > 0) {
                    confirmInvites.push({
                        invite_id: invite.invite_id,
                        station_sn: invite.station_sn,
                        device_sns: devices
                    });
                }
            }
            if (confirmInvites.length > 0) {
                const result = await this.api.confirmInvites(confirmInvites);
                if (result) {
                    this.log.info(`Accepted received invitations`, confirmInvites);
                    this.refreshCloudData();
                }
            }
        }
    }
    async onPushMessage(message) {
        this.emit("push message", message);
        try {
            this.log.debug("Received push message", message);
            try {
                if (message.type === push_1.ServerPushEvent.INVITE_DEVICE && this.config.acceptInvitations) {
                    this.processInvitations();
                }
            }
            catch (error) {
                this.log.error(`Error processing server push notification for device invitation`, error);
            }
            this.getStations().forEach(station => {
                try {
                    station.processPushNotification(message);
                }
                catch (error) {
                    this.log.error(`Error processing push notification for station ${station.getSerial()}`, error);
                }
            });
            this.getDevices().forEach(device => {
                try {
                    device.processPushNotification(message, this.config.eventDurationSeconds);
                }
                catch (error) {
                    this.log.error(`Error processing push notification for device ${device.getSerial()}`, error);
                }
            });
        }
        catch (error) {
            this.log.error("Generic Error:", error);
        }
    }
    async startStationDownload(deviceSN, path, cipherID) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        if (!device.isCamera())
            throw new error_1.NotSupportedError(`This functionality is not implemented or supported by ${device.getSerial()}`);
        if (!station.isDownloading(device)) {
            await station.startDownload(device, path, cipherID);
        }
        else {
            this.log.warn(`The station is already downloading a video for the device ${deviceSN}!`);
        }
    }
    async cancelStationDownload(deviceSN) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        if (!device.isCamera())
            throw new error_1.NotSupportedError(`This functionality is not implemented or supported by ${device.getSerial()}`);
        if (station.isConnected() && station.isDownloading(device)) {
            await station.cancelDownload(device);
        }
        else {
            this.log.warn(`The station isn't downloading a video for the device ${deviceSN}!`);
        }
    }
    getConfig() {
        return this.config;
    }
    async setDeviceProperty(deviceSN, name, value) {
        const device = this.getDevice(deviceSN);
        const station = this.getStation(device.getStationSerial());
        const metadata = device.getPropertyMetadata(name);
        value = (0, utils_1.parseValue)(metadata, value);
        switch (name) {
            case types_1.PropertyName.DeviceEnabled:
                await station.enableDevice(device, value);
                break;
            case types_1.PropertyName.DeviceStatusLed:
                await station.setStatusLed(device, value);
                break;
            case types_1.PropertyName.DeviceAutoNightvision:
                await station.setAutoNightVision(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetection:
                await station.setMotionDetection(device, value);
                break;
            case types_1.PropertyName.DeviceSoundDetection:
                await station.setSoundDetection(device, value);
                break;
            case types_1.PropertyName.DevicePetDetection:
                await station.setPetDetection(device, value);
                break;
            case types_1.PropertyName.DeviceRTSPStream:
                await station.setRTSPStream(device, value);
                break;
            case types_1.PropertyName.DeviceAntitheftDetection:
                await station.setAntiTheftDetection(device, value);
                break;
            case types_1.PropertyName.DeviceLocked:
                await station.lockDevice(device, value);
                break;
            case types_1.PropertyName.DeviceWatermark:
                await station.setWatermark(device, value);
                break;
            case types_1.PropertyName.DeviceLight:
                await station.switchLight(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsEnable:
                await station.setFloodlightLightSettingsEnable(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsBrightnessManual:
                await station.setFloodlightLightSettingsBrightnessManual(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsBrightnessMotion:
                await station.setFloodlightLightSettingsBrightnessMotion(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsBrightnessSchedule:
                await station.setFloodlightLightSettingsBrightnessSchedule(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsMotionTriggered:
                await station.setFloodlightLightSettingsMotionTriggered(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsMotionTriggeredDistance:
                await station.setFloodlightLightSettingsMotionTriggeredDistance(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsMotionTriggeredTimer:
                await station.setFloodlightLightSettingsMotionTriggeredTimer(device, value);
                break;
            case types_1.PropertyName.DeviceMicrophone:
                await station.setMicMute(device, value);
                break;
            case types_1.PropertyName.DeviceSpeaker:
                await station.enableSpeaker(device, value);
                break;
            case types_1.PropertyName.DeviceSpeakerVolume:
                await station.setSpeakerVolume(device, value);
                break;
            case types_1.PropertyName.DeviceAudioRecording:
                await station.setAudioRecording(device, value);
                break;
            case types_1.PropertyName.DevicePowerSource:
                await station.setPowerSource(device, value);
                break;
            case types_1.PropertyName.DevicePowerWorkingMode:
                await station.setPowerWorkingMode(device, value);
                break;
            case types_1.PropertyName.DeviceRecordingEndClipMotionStops:
                await station.setRecordingEndClipMotionStops(device, value);
                break;
            case types_1.PropertyName.DeviceRecordingClipLength:
                await station.setRecordingClipLength(device, value);
                break;
            case types_1.PropertyName.DeviceRecordingRetriggerInterval:
                await station.setRecordingRetriggerInterval(device, value);
                break;
            case types_1.PropertyName.DeviceVideoStreamingQuality:
                await station.setVideoStreamingQuality(device, value);
                break;
            case types_1.PropertyName.DeviceVideoRecordingQuality:
                await station.setVideoRecordingQuality(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionSensitivity:
                await station.setMotionDetectionSensitivity(device, value);
                break;
            case types_1.PropertyName.DeviceMotionTracking:
                await station.setMotionTracking(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionType:
                await station.setMotionDetectionType(device, value);
                break;
            case types_1.PropertyName.DeviceVideoWDR:
                await station.setWDR(device, value);
                break;
            case types_1.PropertyName.DeviceRingtoneVolume:
                await station.setRingtoneVolume(device, value);
                break;
            case types_1.PropertyName.DeviceChimeIndoor:
                await station.enableIndoorChime(device, value);
                break;
            case types_1.PropertyName.DeviceChimeHomebase:
                await station.enableHomebaseChime(device, value);
                break;
            case types_1.PropertyName.DeviceChimeHomebaseRingtoneVolume:
                await station.setHomebaseChimeRingtoneVolume(device, value);
                break;
            case types_1.PropertyName.DeviceChimeHomebaseRingtoneType:
                await station.setHomebaseChimeRingtoneType(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationType:
                await station.setNotificationType(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationPerson:
                await station.setNotificationPerson(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationPet:
                await station.setNotificationPet(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationAllOtherMotion:
                await station.setNotificationAllOtherMotion(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationAllSound:
                await station.setNotificationAllSound(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationCrying:
                await station.setNotificationCrying(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationMotion:
                await station.setNotificationMotion(device, value);
                break;
            case types_1.PropertyName.DeviceNotificationRing:
                await station.setNotificationRing(device, value);
                break;
            case types_1.PropertyName.DeviceChirpVolume:
                await station.setChirpVolume(device, value);
                break;
            case types_1.PropertyName.DeviceChirpTone:
                await station.setChirpTone(device, value);
                break;
            case types_1.PropertyName.DeviceVideoHDR:
                await station.setHDR(device, value);
                break;
            case types_1.PropertyName.DeviceVideoDistortionCorrection:
                await station.setDistortionCorrection(device, value);
                break;
            case types_1.PropertyName.DeviceVideoRingRecord:
                await station.setRingRecord(device, value);
                break;
            case types_1.PropertyName.DeviceRotationSpeed:
                await station.setPanAndTiltRotationSpeed(device, value);
                break;
            case types_1.PropertyName.DeviceNightvision:
                await station.setNightVision(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionRange:
                await station.setMotionDetectionRange(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionRangeStandardSensitivity:
                await station.setMotionDetectionRangeStandardSensitivity(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionRangeAdvancedLeftSensitivity:
                await station.setMotionDetectionRangeAdvancedLeftSensitivity(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionRangeAdvancedMiddleSensitivity:
                await station.setMotionDetectionRangeAdvancedMiddleSensitivity(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionRangeAdvancedRightSensitivity:
                await station.setMotionDetectionRangeAdvancedRightSensitivity(device, value);
                break;
            case types_1.PropertyName.DeviceMotionDetectionTestMode:
                await station.setMotionDetectionTestMode(device, value);
                break;
            case types_1.PropertyName.DeviceMotionTrackingSensitivity:
                await station.setMotionTrackingSensitivity(device, value);
                break;
            case types_1.PropertyName.DeviceMotionAutoCruise:
                await station.setMotionAutoCruise(device, value);
                break;
            case types_1.PropertyName.DeviceMotionOutOfViewDetection:
                await station.setMotionOutOfViewDetection(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsColorTemperatureManual:
                await station.setLightSettingsColorTemperatureManual(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsColorTemperatureMotion:
                await station.setLightSettingsColorTemperatureMotion(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsColorTemperatureSchedule:
                await station.setLightSettingsColorTemperatureSchedule(device, value);
                break;
            case types_1.PropertyName.DeviceLightSettingsMotionActivationMode:
                await station.setLightSettingsMotionActivationMode(device, value);
                break;
            case types_1.PropertyName.DeviceVideoNightvisionImageAdjustment:
                await station.setVideoNightvisionImageAdjustment(device, value);
                break;
            case types_1.PropertyName.DeviceVideoColorNightvision:
                await station.setVideoColorNightvision(device, value);
                break;
            case types_1.PropertyName.DeviceAutoCalibration:
                await station.setAutoCalibration(device, value);
                break;
            default:
                if (!Object.values(types_1.PropertyName).includes(name))
                    throw new error_1.ReadOnlyPropertyError(`Property ${name} is read only`);
                throw new error_2.InvalidPropertyError(`Device ${deviceSN} has no writable property named ${name}`);
        }
    }
    async setStationProperty(stationSN, name, value) {
        const station = this.getStation(stationSN);
        const metadata = station.getPropertyMetadata(name);
        value = (0, utils_1.parseValue)(metadata, value);
        switch (name) {
            case types_1.PropertyName.StationGuardMode:
                await station.setGuardMode(value);
                break;
            case types_1.PropertyName.StationAlarmTone:
                await station.setStationAlarmTone(value);
                break;
            case types_1.PropertyName.StationAlarmVolume:
                await station.setStationAlarmRingtoneVolume(value);
                break;
            case types_1.PropertyName.StationPromptVolume:
                await station.setStationPromptVolume(value);
                break;
            case types_1.PropertyName.StationNotificationSwitchModeApp:
                await station.setStationNotificationSwitchMode(types_1.NotificationSwitchMode.APP, value);
                break;
            case types_1.PropertyName.StationNotificationSwitchModeGeofence:
                await station.setStationNotificationSwitchMode(types_1.NotificationSwitchMode.GEOFENCE, value);
                break;
            case types_1.PropertyName.StationNotificationSwitchModeSchedule:
                await station.setStationNotificationSwitchMode(types_1.NotificationSwitchMode.SCHEDULE, value);
                break;
            case types_1.PropertyName.StationNotificationSwitchModeKeypad:
                await station.setStationNotificationSwitchMode(types_1.NotificationSwitchMode.KEYPAD, value);
                break;
            case types_1.PropertyName.StationNotificationStartAlarmDelay:
                await station.setStationNotificationStartAlarmDelay(value);
                break;
            case types_1.PropertyName.StationTimeFormat:
                await station.setStationTimeFormat(value);
                break;
            case types_1.PropertyName.StationSwitchModeWithAccessCode:
                await station.setStationSwitchModeWithAccessCode(value);
                break;
            case types_1.PropertyName.StationAutoEndAlarm:
                await station.setStationAutoEndAlarm(value);
                break;
            case types_1.PropertyName.StationTurnOffAlarmWithButton:
                await station.setStationTurnOffAlarmWithButton(value);
                break;
            default:
                if (!Object.values(types_1.PropertyName).includes(name))
                    throw new error_1.ReadOnlyPropertyError(`Property ${name} is read only`);
                throw new error_2.InvalidPropertyError(`Station ${stationSN} has no writable property named ${name}`);
        }
    }
    onStartStationLivestream(station, channel, metadata, videostream, audiostream) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station livestream start", station, device, metadata, videostream, audiostream);
        }
        catch (error) {
            this.log.error(`Station start livestream error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStopStationLivestream(station, channel) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station livestream stop", station, device);
        }
        catch (error) {
            this.log.error(`Station stop livestream error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStartStationRTSPLivestream(station, channel) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station rtsp livestream start", station, device);
        }
        catch (error) {
            this.log.error(`Station start rtsp livestream error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStopStationRTSPLivestream(station, channel) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station rtsp livestream stop", station, device);
        }
        catch (error) {
            this.log.error(`Station stop rtsp livestream error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStationStartDownload(station, channel, metadata, videoStream, audioStream) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station download start", station, device, metadata, videoStream, audioStream);
        }
        catch (error) {
            this.log.error(`Station start download error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStationFinishDownload(station, channel) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station download finish", station, device);
        }
        catch (error) {
            this.log.error(`Station finish download error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    async onStationCommandResult(station, result) {
        try {
            this.emit("station command result", station, result);
            if (result.return_code === 0 && result.command_type !== types_2.CommandType.CMD_CAMERA_INFO && result.command_type !== types_2.CommandType.CMD_PING) {
                await this.api.updateDeviceInfo();
                if (station.isConnected() && station.getDeviceType() !== types_1.DeviceType.DOORBELL)
                    await station.getCameraInfo();
            }
        }
        catch (error) {
            this.log.error(`Station command result error (station: ${station.getSerial()})`, error);
        }
    }
    onStationRtspUrl(station, channel, value, modified) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            this.emit("station rtsp url", station, device, value, modified);
            device.setCustomPropertyValue(types_1.PropertyName.DeviceRTSPStreamUrl, { value: value, timestamp: modified });
        }
        catch (error) {
            this.log.error(`Station rtsp url error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStationGuardMode(station, guardMode) {
        this.emit("station guard mode", station, guardMode);
    }
    onStationCurrentMode(station, currentMode) {
        this.emit("station current mode", station, currentMode);
    }
    onStationPropertyChanged(station, name, value) {
        this.emit("station property changed", station, name, value);
    }
    onStationPropertyRenewed(station, name, value) {
        this.emit("station property renewed", station, name, value);
    }
    onStationRawPropertyChanged(station, type, value, modified) {
        this.emit("station raw property changed", station, type, value, modified);
    }
    onStationRawPropertyRenewed(station, type, value, modified) {
        this.emit("station raw property renewed", station, type, value, modified);
    }
    onStationAlarmEvent(station, alarmEvent) {
        this.emit("station alarm event", station, alarmEvent);
    }
    onDevicePropertyChanged(device, name, value) {
        try {
            this.emit("device property changed", device, name, value);
            if (name === types_1.PropertyName.DeviceRTSPStream && value.value === true && (device.getPropertyValue(types_1.PropertyName.DeviceRTSPStreamUrl) === undefined || (device.getPropertyValue(types_1.PropertyName.DeviceRTSPStreamUrl) !== undefined && device.getPropertyValue(types_1.PropertyName.DeviceRTSPStreamUrl).value === ""))) {
                this.getStation(device.getStationSerial()).setRTSPStream(device, true);
            }
            else if (name === types_1.PropertyName.DeviceRTSPStream && value.value === false) {
                device.setCustomPropertyValue(types_1.PropertyName.DeviceRTSPStreamUrl, { value: "", timestamp: new Date().getTime() });
            }
        }
        catch (error) {
            this.log.error(`Device property changed error (device: ${device.getSerial()} name: ${name})`, error);
        }
    }
    onDevicePropertyRenewed(device, name, value) {
        this.emit("device property renewed", device, name, value);
    }
    onDeviceRawPropertyChanged(device, type, value, modified) {
        this.emit("device raw property changed", device, type, value, modified);
    }
    onDeviceRawPropertyRenewed(device, type, value, modified) {
        this.emit("device raw property renewed", device, type, value, modified);
    }
    onDeviceCryingDetected(device, state) {
        this.emit("device crying detected", device, state);
    }
    onDeviceSoundDetected(device, state) {
        this.emit("device sound detected", device, state);
    }
    onDevicePetDetected(device, state) {
        this.emit("device pet detected", device, state);
    }
    onDeviceMotionDetected(device, state) {
        this.emit("device motion detected", device, state);
    }
    onDevicePersonDetected(device, state, person) {
        this.emit("device person detected", device, state, person);
    }
    onDeviceRings(device, state) {
        this.emit("device rings", device, state);
    }
    onDeviceLocked(device, state) {
        this.emit("device locked", device, state);
    }
    onDeviceOpen(device, state) {
        this.emit("device open", device, state);
    }
    onDeviceReady(device) {
        try {
            if (device.getPropertyValue(types_1.PropertyName.DeviceRTSPStream) !== undefined && device.getPropertyValue(types_1.PropertyName.DeviceRTSPStream).value === true) {
                this.getStation(device.getStationSerial()).setRTSPStream(device, true);
            }
        }
        catch (error) {
            this.log.error(`Device ready error (device: ${device.getSerial()})`, error);
        }
    }
    onStationRuntimeState(station, channel, batteryLevel, temperature, modified) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            if (device.hasProperty(types_1.PropertyName.DeviceBattery)) {
                const metadataBattery = device.getPropertyMetadata(types_1.PropertyName.DeviceBattery);
                device.updateRawProperty(metadataBattery.key, { value: batteryLevel.toString(), timestamp: modified });
            }
            if (device.hasProperty(types_1.PropertyName.DeviceBatteryTemp)) {
                const metadataBatteryTemperature = device.getPropertyMetadata(types_1.PropertyName.DeviceBatteryTemp);
                device.updateRawProperty(metadataBatteryTemperature.key, { value: temperature.toString(), timestamp: modified });
            }
        }
        catch (error) {
            this.log.error(`Station runtime state error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStationChargingState(station, channel, chargeType, batteryLevel, modified) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            if (device.hasProperty(types_1.PropertyName.DeviceBattery)) {
                const metadataBattery = device.getPropertyMetadata(types_1.PropertyName.DeviceBattery);
                device.updateRawProperty(metadataBattery.key, { value: batteryLevel.toString(), timestamp: modified });
            }
            if (device.hasProperty(types_1.PropertyName.DeviceChargingStatus)) {
                const metadataChargingStatus = device.getPropertyMetadata(types_1.PropertyName.DeviceChargingStatus);
                device.updateRawProperty(metadataChargingStatus.key, { value: chargeType.toString(), timestamp: modified });
            }
        }
        catch (error) {
            this.log.error(`Station charging state error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onStationWifiRssi(station, channel, rssi, modified) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            if (device.hasProperty(types_1.PropertyName.DeviceWifiRSSI)) {
                const metadataWifiRssi = device.getPropertyMetadata(types_1.PropertyName.DeviceWifiRSSI);
                device.updateRawProperty(metadataWifiRssi.key, { value: rssi.toString(), timestamp: modified });
            }
        }
        catch (error) {
            this.log.error(`Station wifi rssi error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
    onCaptchaRequest(id, captcha) {
        this.emit("captcha request", id, captcha);
    }
    onFloodlightManualSwitch(station, channel, enabled, modified) {
        try {
            const device = this.getStationDevice(station.getSerial(), channel);
            if (device.hasProperty(types_1.PropertyName.DeviceLight)) {
                const metadataLight = device.getPropertyMetadata(types_1.PropertyName.DeviceLight);
                device.updateRawProperty(metadataLight.key, { value: enabled === true ? "1" : "0", timestamp: modified });
            }
        }
        catch (error) {
            this.log.error(`Station floodlight manual switch error (station: ${station.getSerial()} channel: ${channel})`, error);
        }
    }
}
exports.EufySecurity = EufySecurity;
//# sourceMappingURL=eufysecurity.js.map